﻿namespace FlickRSearcher.Utility
{
  using System.Collections.Generic;
  using System.IO;
  using System.Linq;
  using System.Net;
  using System.Xml.Linq;

  internal class FlickrPhotoResultsPage
  {
    public FlickrPhotoResultsPage(Stream stream)
    {
      XElement result =
        XElement.Load(stream).DescendantsAndSelf("photos").First();

      this.PageNo = (int)result.Attribute("page");
      this.Pages = (long)result.Attribute("pages");
      this.PerPage = (int)result.Attribute("perpage");
      this.Total = (long)result.Attribute("total");

      LoadPhotos(result);

      stream.Close();
    }    
    public int PageNo { get; private set; }
    public long Pages { get; private set; }
    public int PerPage { get; private set; }
    public long Total { get; private set; }

    public IEnumerable<string> PhotoUris
    {
      get
      {
        return (Photos.Select(p => p.ImageUri.ToString()));
      }
    }
    public IEnumerable<FlickrPhotoResult> Photos
    {
      get
      {
        return (this.photos);
      }
    }
    IEnumerable<FlickrPhotoResult> photos;

    void LoadPhotos(XElement result)
    {
      photos = 
        result.DescendantsAndSelf("photo").Select(
          (element) =>
          {
            return (new FlickrPhotoResult(element));
          });
    }    
    public static FlickrPhotoResultsPage FromWebResponse(WebResponse response)
    {
      FlickrPhotoResultsPage result = null;

      using (response)
      {
        using (Stream stream = response.GetResponseStream())
        {
          result = new FlickrPhotoResultsPage(stream);
          stream.Close();
        }
        response.Close();
      }
      return (result);
    }
    public static FlickrPhotoResultsPage FromStream(Stream stream)
    {
      return (new FlickrPhotoResultsPage(stream));
    }

  }
}
