﻿namespace FlickRSearcher.ViewModels
{
  using System;
  using System.IO;
  using System.Reactive.Linq;
  using System.Windows.Media.Imaging;
  using FlickRSearcher.Utility;

  public enum ImageViewModelState
  {
    Loading,
    Errored,
    Cancelled,
    Loaded
  }
  public class ImageViewModel : PropertyChangedNotification
  {
    public ImageViewModel(string title, Uri uri)
    {
      this.Title = title;
      this.Uri = uri;
      this.InternalVisualState = ImageViewModelState.Loading;
      LoadImageAsync();
    }
    public string Title { get; private set; }

    public Uri Uri { get; private set; }

    public BitmapImage ImageSource
    {
      get
      {
        return (_ImageSource);
      }
      set
      {
        _ImageSource = value;
        RaisePropertyChanged("ImageSource");
      }
    }
    BitmapImage _ImageSource;

    /// <summary>
    /// This is a bit of a hack as it gives the idea that we have one visual
    /// state when in fact there might be groups of visual states and this
    /// doesn't really reflect that but it serves my purpose here.
    /// </summary>
    public string VisualState
    {
      get
      {
        return (this.InternalVisualState.ToString());
      }
    }
    public ImageViewModelState InternalVisualState
    {
      get
      {
        return (this.internalVisualState);
      }
      set
      {
        this.internalVisualState = value;
        RaisePropertyChanged("VisualState");
      }
    }
    ImageViewModelState internalVisualState;

    void LoadImageAsync()
    {
      ObservableWebRequest
        .CreateDeferred(this.Uri.ToString())
        .Select(response => response.ReadResponseToMemoryStream())
        .ObserveOnDispatcher()
        .Subscribe(
          SetImage,
          ex => this.InternalVisualState = ImageViewModelState.Errored,
          () => this.InternalVisualState = ImageViewModelState.Cancelled);
    }
    void SetImage(Stream image)
    {
      this.InternalVisualState = ImageViewModelState.Loaded;
      BitmapImage bitmapImage = new BitmapImage();
      bitmapImage.SetSource(image);
      this.ImageSource = bitmapImage;
    }
  }
}